@@ -2,16 +2,16 @@
     Boost.Wave: A Standard compliant C++ preprocessor library
 
     SLex (Spirit Lex) based C++ lexer
-    
+
     http://www.boost.org/
 
-    Copyright (c) 2001-2008 Hartmut Kaiser. Distributed under the Boost 
-    Software License, Version 1.0. (See accompanying file 
+    Copyright (c) 2001-2012 Hartmut Kaiser. Distributed under the Boost
+    Software License, Version 1.0. (See accompanying file
     LICENSE_1_0.txt or copy at http://www.boost.org/LICENSE_1_0.txt)
 =============================================================================*/
 
-#if !defined(SLEX_LEXER_HPP_5E8E1DF0_BB41_4938_B7E5_A4BB68222FF6_INCLUDED)
-#define SLEX_LEXER_HPP_5E8E1DF0_BB41_4938_B7E5_A4BB68222FF6_INCLUDED
+#if !defined(BOOST_SLEX_LEXER_HPP_5E8E1DF0_BB41_4938_B7E5_A4BB68222FF6_INCLUDED)
+#define BOOST_SLEX_LEXER_HPP_5E8E1DF0_BB41_4938_B7E5_A4BB68222FF6_INCLUDED
 
 #include <string>
 #if defined(BOOST_SPIRIT_DEBUG)
@@ -19,7 +19,7 @@
 #endif // defined(BOOST_SPIRIT_DEBUG)
 
 #include <boost/assert.hpp>
-#include <boost/spirit/core.hpp>
+#include <boost/spirit/include/classic_core.hpp>
 
 #include <boost/wave/wave_config.hpp>
 #include <boost/wave/language_support.hpp>
@@ -57,29 +57,31 @@ namespace lexer {
 #endif
 #define INIT_DATA_CPP_SIZE          15
 #define INIT_DATA_PP_NUMBER_SIZE    2
+#define INIT_DATA_CPP0X_SIZE        15
+#define INIT_DATA_CPP2A_SIZE        10
 
 ///////////////////////////////////////////////////////////////////////////////
-// 
-//  encapsulation of the boost::spirit::slex based cpp lexer
+//
+//  encapsulation of the boost::spirit::classic::slex based cpp lexer
 //
 ///////////////////////////////////////////////////////////////////////////////
 
 ///////////////////////////////////////////////////////////////////////////////
-//  The following lexer_base class was necessary to workaround a CodeWarrior 
+//  The following lexer_base class was necessary to workaround a CodeWarrior
 //  bug (at least up to CW V9.5).
 template <typename IteratorT, typename PositionT>
-class lexer_base 
-:   public boost::spirit::lexer<
+class lexer_base
+:   public boost::spirit::classic::lexer<
         boost::wave::util::position_iterator<IteratorT, PositionT> >
 {
 protected:
-    typedef boost::wave::util::position_iterator<IteratorT, PositionT> 
+    typedef boost::wave::util::position_iterator<IteratorT, PositionT>
         iterator_type;
     typedef typename std::iterator_traits<IteratorT>::value_type  char_type;
-    typedef boost::spirit::lexer<iterator_type>                   base_type;
+    typedef boost::spirit::classic::lexer<iterator_type> base_type;
 
     lexer_base();
-    
+
 // initialization data (regular expressions for the token definitions)
     struct lexer_data {
         token_id tokenid;                       // token data
@@ -91,16 +93,16 @@ protected:
 
 ///////////////////////////////////////////////////////////////////////////////
 template <typename IteratorT, typename PositionT>
-class lexer 
+class lexer
 :   public lexer_base<IteratorT, PositionT>
 {
 public:
     typedef boost::wave::cpplexer::slex_token<PositionT>  token_type;
-    
+
     void init_dfa(boost::wave::language_support language);
 
 // get time of last compilation
-    static std::time_t get_compilation_time() 
+    static std::time_t get_compilation_time()
         { return compilation_time.get_time(); }
 
 // helper for calculation of the time of last compilation
@@ -112,6 +114,8 @@ private:
     static typename base_type::lexer_data const init_data[INIT_DATA_SIZE];          // common patterns
     static typename base_type::lexer_data const init_data_cpp[INIT_DATA_CPP_SIZE];  // C++ only patterns
     static typename base_type::lexer_data const init_data_pp_number[INIT_DATA_PP_NUMBER_SIZE];  // pp-number only patterns
+    static typename base_type::lexer_data const init_data_cpp0x[INIT_DATA_CPP0X_SIZE];  // C++0X only patterns
+    static typename base_type::lexer_data const init_data_cpp2a[INIT_DATA_CPP2A_SIZE];  // C++2A only patterns
 };
 
 ///////////////////////////////////////////////////////////////////////////////
@@ -124,7 +128,7 @@ private:
 #define BLANK               "[ \\t]"
 #define CCOMMENT            \
     Q("/") Q("*") "[^*]*" Q("*") "+" "(" "[^/*][^*]*" Q("*") "+" ")*" Q("/")
-        
+
 #define PPSPACE             "(" BLANK OR CCOMMENT ")*"
 
 #define OCTALDIGIT          "[0-7]"
@@ -137,19 +141,20 @@ private:
 
 #define INTEGER             \
     "(" "(0x|0X)" HEXDIGIT "+" OR "0" OCTALDIGIT "*" OR "[1-9]" DIGIT "*" ")"
-            
+
 #define INTEGER_SUFFIX      "(" "[uU][lL]?|[lL][uU]?" ")"
 #if BOOST_WAVE_SUPPORT_MS_EXTENSIONS != 0
-#define LONGINTEGER_SUFFIX  "(" "[uU]" "(" "[lL][lL]" ")" OR \
-                                "(" "[lL][lL]" ")" "[uU]" "?" OR \
+#define LONGINTEGER_SUFFIX  "(" "[uU]" "(" "ll" OR "LL" ")" OR \
+                                "(" "ll" OR "LL" ")" "[uU]" "?" OR \
                                 "i64" \
-                            ")" 
+                            ")"
 #else
-#define LONGINTEGER_SUFFIX  "(" "[uU]" "(" "[lL][lL]" ")" OR \
-                            "(" "[lL][lL]" ")" "[uU]" "?" ")"
+#define LONGINTEGER_SUFFIX  "(" "[uU]" "(" "ll" OR "LL" ")" OR \
+                            "(" "ll" OR "LL" ")" "[uU]" "?" ")"
 #endif
 #define FLOAT_SUFFIX        "(" "[fF][lL]?" OR "[lL][fF]?" ")"
 #define CHAR_SPEC           "L?"
+#define EXTCHAR_SPEC        "(" "[uU]" OR "u8" ")"
 
 #define BACKSLASH           "(" Q("\\") OR TRI(Q("/")) ")"
 #define ESCAPESEQ           "(" BACKSLASH "(" \
@@ -162,7 +167,7 @@ private:
 #define UNIVERSALCHAR       "(" BACKSLASH "(" \
                                 "u" HEXQUAD OR \
                                 "U" HEXQUAD HEXQUAD \
-                            "))" 
+                            "))"
 
 #define POUNDDEF            "(" "#" OR TRI("=") OR Q("%:") ")"
 #define NEWLINEDEF          "(" "\n" OR "\r" OR "\r\n" ")"
@@ -194,8 +199,8 @@ private:
 ///////////////////////////////////////////////////////////////////////////////
 // common C++/C99 token definitions
 template <typename IteratorT, typename PositionT>
-typename lexer_base<IteratorT, PositionT>::lexer_data const 
-lexer<IteratorT, PositionT>::init_data[INIT_DATA_SIZE] = 
+typename lexer_base<IteratorT, PositionT>::lexer_data const
+lexer<IteratorT, PositionT>::init_data[INIT_DATA_SIZE] =
 {
     TOKEN_DATA(AND, "&"),
     TOKEN_DATA(ANDAND, "&&"),
@@ -362,24 +367,24 @@ lexer<IteratorT, PositionT>::init_data[INIT_DATA_SIZE] =
 //  TOKEN_DATA(HEXAINT, "(0x|0X)" HEXDIGIT "+" INTEGER_SUFFIX "?"),
     TOKEN_DATA(LONGINTLIT, INTEGER LONGINTEGER_SUFFIX),
     TOKEN_DATA(INTLIT, INTEGER INTEGER_SUFFIX "?"),
-    TOKEN_DATA(FLOATLIT, 
-        "(" DIGIT "*" Q(".") DIGIT "+" OR DIGIT "+" Q(".") ")" 
+    TOKEN_DATA(FLOATLIT,
+        "(" DIGIT "*" Q(".") DIGIT "+" OR DIGIT "+" Q(".") ")"
         EXPONENT "?" FLOAT_SUFFIX "?" OR
         DIGIT "+" EXPONENT FLOAT_SUFFIX "?"),
+    TOKEN_DATA(CCOMMENT, CCOMMENT),
+    TOKEN_DATA(CPPCOMMENT, Q("/") Q("/[^\\n\\r]*") NEWLINEDEF ),
+    TOKEN_DATA(CHARLIT, CHAR_SPEC "'"
+                "(" ESCAPESEQ OR UNIVERSALCHAR OR "[^\\n\\r\\\\']" ")+" "'"),
+    TOKEN_DATA(STRINGLIT, CHAR_SPEC Q("\"")
+                "(" ESCAPESEQ OR UNIVERSALCHAR OR "[^\\n\\r\\\\\"]" ")*" Q("\"")),
 #if BOOST_WAVE_USE_STRICT_LEXER != 0
     TOKEN_DATA(IDENTIFIER, "([a-zA-Z_]" OR UNIVERSALCHAR ")([a-zA-Z0-9_]" OR UNIVERSALCHAR ")*"),
 #else
     TOKEN_DATA(IDENTIFIER, "([a-zA-Z_$]" OR UNIVERSALCHAR ")([a-zA-Z0-9_$]" OR UNIVERSALCHAR ")*"),
 #endif
-    TOKEN_DATA(CCOMMENT, CCOMMENT),
-    TOKEN_DATA(CPPCOMMENT, Q("/") Q("/[^\\n\\r]*") NEWLINEDEF ),
-    TOKEN_DATA(CHARLIT, CHAR_SPEC "'" 
-                "(" ESCAPESEQ OR "[^\\n\\r']" OR UNIVERSALCHAR ")+" "'"),
-    TOKEN_DATA(STRINGLIT, CHAR_SPEC Q("\"") 
-                "(" ESCAPESEQ OR "[^\\n\\r\"]" OR UNIVERSALCHAR ")*" Q("\"")),
     TOKEN_DATA(SPACE, "[ \t\v\f]+"),
 //    TOKEN_DATA(SPACE2, "[\\v\\f]+"),
-    TOKEN_DATA(CONTLINE, "\\" "\\n"), 
+    TOKEN_DATA(CONTLINE, Q("\\") "\n"),
     TOKEN_DATA(NEWLINE, NEWLINEDEF),
     TOKEN_DATA(POUND_POUND, "##"),
     TOKEN_DATA(POUND_POUND_ALT, Q("%:") Q("%:")),
@@ -395,8 +400,8 @@ lexer<IteratorT, PositionT>::init_data[INIT_DATA_SIZE] =
 ///////////////////////////////////////////////////////////////////////////////
 // C++ only token definitions
 template <typename IteratorT, typename PositionT>
-typename lexer_base<IteratorT, PositionT>::lexer_data const 
-lexer<IteratorT, PositionT>::init_data_cpp[INIT_DATA_CPP_SIZE] = 
+typename lexer_base<IteratorT, PositionT>::lexer_data const
+lexer<IteratorT, PositionT>::init_data_cpp[INIT_DATA_CPP_SIZE] =
 {
     TOKEN_DATA(AND_ALT, "bitand"),
     TOKEN_DATA(ANDASSIGN_ALT, "and_eq"),
@@ -421,13 +426,66 @@ lexer<IteratorT, PositionT>::init_data_cpp[INIT_DATA_CPP_SIZE] =
 ///////////////////////////////////////////////////////////////////////////////
 // C++ only token definitions
 template <typename IteratorT, typename PositionT>
-typename lexer_base<IteratorT, PositionT>::lexer_data const 
-lexer<IteratorT, PositionT>::init_data_pp_number[INIT_DATA_PP_NUMBER_SIZE] = 
+typename lexer_base<IteratorT, PositionT>::lexer_data const
+lexer<IteratorT, PositionT>::init_data_pp_number[INIT_DATA_PP_NUMBER_SIZE] =
 {
     TOKEN_DATA(PP_NUMBER, PP_NUMBERDEF),
     { token_id(0) }       // this should be the last entry
 };
 
+///////////////////////////////////////////////////////////////////////////////
+// C++11 only token definitions
+
+constexpr token_id T_EXTCHARLIT = T_CHARLIT | AltTokenType;
+constexpr token_id T_EXTSTRINGLIT = T_STRINGLIT | AltTokenType;
+constexpr token_id T_EXTRAWSTRINGLIT = T_RAWSTRINGLIT | AltTokenType;
+
+template <typename IteratorT, typename PositionT>
+typename lexer_base<IteratorT, PositionT>::lexer_data const
+lexer<IteratorT, PositionT>::init_data_cpp0x[INIT_DATA_CPP0X_SIZE] =
+{
+    TOKEN_DATA(EXTCHARLIT, EXTCHAR_SPEC "'"
+                "(" ESCAPESEQ OR UNIVERSALCHAR OR "[^\\n\\r\\\\']" ")+" "'"),
+    TOKEN_DATA(EXTSTRINGLIT, EXTCHAR_SPEC Q("\"")
+                "(" ESCAPESEQ OR UNIVERSALCHAR OR "[^\\n\\r\\\\\"]" ")*" Q("\"")),
+    TOKEN_DATA(RAWSTRINGLIT, CHAR_SPEC "R" Q("\"")
+                "(" ESCAPESEQ OR UNIVERSALCHAR OR "[^\\\\\"]" ")*" Q("\"")),
+    TOKEN_DATA(EXTRAWSTRINGLIT, EXTCHAR_SPEC "R" Q("\"")
+                "(" ESCAPESEQ OR UNIVERSALCHAR OR "[^\\\\\"]" ")*" Q("\"")),
+    TOKEN_DATA(ALIGNAS, "alignas"),
+    TOKEN_DATA(ALIGNOF, "alignof"),
+    TOKEN_DATA(CHAR16_T, "char16_t"),
+    TOKEN_DATA(CHAR32_T, "char32_t"),
+    TOKEN_DATA(CONSTEXPR, "constexpr"),
+    TOKEN_DATA(DECLTYPE, "decltype"),
+    TOKEN_DATA(NOEXCEPT, "noexcept"),
+    TOKEN_DATA(NULLPTR, "nullptr"),
+    TOKEN_DATA(STATICASSERT, "static_assert"),
+    TOKEN_DATA(THREADLOCAL, "thread_local"),
+    { token_id(0) }       // this should be the last entry
+};
+
+///////////////////////////////////////////////////////////////////////////////
+// C++20 only token definitions
+
+template <typename IteratorT, typename PositionT>
+typename lexer_base<IteratorT, PositionT>::lexer_data const
+lexer<IteratorT, PositionT>::init_data_cpp2a[INIT_DATA_CPP2A_SIZE] =
+{
+    TOKEN_DATA(CHAR8_T, "char8_t"),
+    TOKEN_DATA(CONCEPT, "concept"),
+    TOKEN_DATA(CONSTEVAL, "consteval"),
+    TOKEN_DATA(CONSTINIT, "constinit"),
+    TOKEN_DATA(CO_AWAIT, "co_await"),
+    TOKEN_DATA(CO_RETURN, "co_return"),
+    TOKEN_DATA(CO_YIELD, "co_yield"),
+    TOKEN_DATA(REQUIRES, "requires"),
+    TOKEN_DATA(SPACESHIP, "<=>"),
+
+    { token_id(0) }       // this should be the last entry
+};
+
+
 ///////////////////////////////////////////////////////////////////////////////
 //  undefine macros, required for regular expression definitions
 #undef INCLUDEDEF
@@ -446,9 +504,9 @@ lexer<IteratorT, PositionT>::init_data_pp_number[INIT_DATA_PP_NUMBER_SIZE] =
 #undef INTEGER
 #undef FLOAT_SUFFIX
 #undef CHAR_SPEC
-#undef BACKSLASH    
-#undef ESCAPESEQ    
-#undef HEXQUAD      
+#undef BACKSLASH
+#undef ESCAPESEQ
+#undef HEXQUAD
 #undef UNIVERSALCHAR
 #undef PP_NUMBERDEF
 
@@ -463,7 +521,7 @@ lexer<IteratorT, PositionT>::init_data_pp_number[INIT_DATA_PP_NUMBER_SIZE] =
 // initialize cpp lexer with token data
 template <typename IteratorT, typename PositionT>
 inline
-lexer_base<IteratorT, PositionT>::lexer_base() 
+lexer_base<IteratorT, PositionT>::lexer_base()
 :   base_type(NUM_LEXER_STATES)
 {
 }
@@ -478,23 +536,46 @@ lexer<IteratorT, PositionT>::init_dfa(boost::wave::language_support lang)
 // if pp-numbers should be preferred, insert the corresponding rule first
     if (boost::wave::need_prefer_pp_numbers(lang)) {
         for (int j = 0; 0 != init_data_pp_number[j].tokenid; ++j) {
-            this->register_regex(init_data_pp_number[j].tokenregex, 
-                init_data_pp_number[j].tokenid, init_data_pp_number[j].tokencb, 
+            this->register_regex(init_data_pp_number[j].tokenregex,
+                init_data_pp_number[j].tokenid, init_data_pp_number[j].tokencb,
                 init_data_pp_number[j].lexerstate);
         }
     }
-        
-// if in C99 mode, some of the keywords are not valid    
+
+// if in C99 mode, some of the keywords are not valid
     if (!boost::wave::need_c99(lang)) {
         for (int j = 0; 0 != init_data_cpp[j].tokenid; ++j) {
-            this->register_regex(init_data_cpp[j].tokenregex, 
-                init_data_cpp[j].tokenid, init_data_cpp[j].tokencb, 
+            this->register_regex(init_data_cpp[j].tokenregex,
+                init_data_cpp[j].tokenid, init_data_cpp[j].tokencb,
                 init_data_cpp[j].lexerstate);
         }
     }
-    
+
+// if in C++0x mode, add appropriate keywords
+#if BOOST_WAVE_SUPPORT_CPP0X != 0
+    if (boost::wave::need_cpp0x(lang) || boost::wave::need_cpp2a(lang)) {
+        for (int j = 0; 0 != init_data_cpp0x[j].tokenid; ++j) {
+            this->register_regex(init_data_cpp0x[j].tokenregex,
+                init_data_cpp0x[j].tokenid, init_data_cpp0x[j].tokencb,
+                init_data_cpp0x[j].lexerstate);
+        }
+    }
+#endif
+
+    // if in C++2a mode, add those keywords
+#if BOOST_WAVE_SUPPORT_CPP2A != 0
+        if (wave::need_cpp2a(lang)) {
+            for (int j = 0; 0 != init_data_cpp2a[j].tokenid; ++j) {
+                this->register_regex(init_data_cpp2a[j].tokenregex,
+                                     init_data_cpp2a[j].tokenid,
+                                     init_data_cpp2a[j].tokencb,
+                                     init_data_cpp2a[j].lexerstate);
+        }
+    }
+#endif
+
     for (int i = 0; 0 != init_data[i].tokenid; ++i) {
-        this->register_regex(init_data[i].tokenregex, init_data[i].tokenid, 
+        this->register_regex(init_data[i].tokenregex, init_data[i].tokenid,
             init_data[i].tokencb, init_data[i].lexerstate);
     }
 }
@@ -502,28 +583,28 @@ lexer<IteratorT, PositionT>::init_dfa(boost::wave::language_support lang)
 ///////////////////////////////////////////////////////////////////////////////
 // get time of last compilation of this file
 template <typename IteratorT, typename PositionT>
-boost::wave::util::time_conversion_helper 
+boost::wave::util::time_conversion_helper
     lexer<IteratorT, PositionT>::compilation_time(__DATE__ " " __TIME__);
 
 ///////////////////////////////////////////////////////////////////////////////
 }   // namespace lexer
 
 ///////////////////////////////////////////////////////////////////////////////
-//  
+//
 template <typename IteratorT, typename PositionT>
-inline void 
-init_lexer (lexer::lexer<IteratorT, PositionT> &lexer, 
+inline void
+init_lexer (lexer::lexer<IteratorT, PositionT> &lexer,
     boost::wave::language_support language, bool force_reinit = false)
 {
     if (lexer.has_compiled_dfa())
         return;     // nothing to do
-        
+
     using std::ifstream;
     using std::ofstream;
     using std::ios;
     using std::cerr;
     using std::endl;
-    
+
 ifstream dfa_in("wave_slex_lexer.dfa", ios::in|ios::binary);
 
     lexer.init_dfa(language);
@@ -549,13 +630,13 @@ ifstream dfa_in("wave_slex_lexer.dfa", ios::in|ios::binary);
 }
 
 ///////////////////////////////////////////////////////////////////////////////
-//  
+//
 //  lex_functor
 //
 ///////////////////////////////////////////////////////////////////////////////
 
 template <typename IteratorT, typename PositionT = wave::util::file_position_type>
-class slex_functor 
+class slex_functor
 :   public slex_input_interface<
         typename lexer::lexer<IteratorT, PositionT>::token_type
     >
@@ -568,19 +649,19 @@ public:
     typedef BOOST_WAVE_STRINGTYPE                                   string_type;
     typedef typename lexer::lexer<IteratorT, PositionT>::token_type token_type;
 
-    slex_functor(IteratorT const &first_, IteratorT const &last_, 
+    slex_functor(IteratorT const &first_, IteratorT const &last_,
             PositionT const &pos_, boost::wave::language_support language_)
     :   first(first_, last_, pos_), language(language_), at_eof(false)
     {
         // initialize lexer dfa tables
-        init_lexer(lexer, language_);  
+        init_lexer(lexer, language_);
     }
     virtual ~slex_functor() {}
 
 // get the next token from the input stream
-    token_type get()
+    token_type& get(token_type& result) BOOST_OVERRIDE
     {
-        token_type token;
+        using namespace cpplexer::slex::lexer;
 
         if (!at_eof) {
             do {
@@ -593,69 +674,78 @@ public:
                     id = T_EOF;     // end of input reached
 
             string_type token_val(value.c_str());
-            
-                if (T_CONTLINE != id) {
-                //  The cast should avoid spurious warnings about missing case labels 
+
+                if (boost::wave::need_emit_contnewlines(language) ||
+                    T_CONTLINE != id)
+                {
+                //  The cast should avoid spurious warnings about missing case labels
                 //  for the other token ids's.
-                    switch (static_cast<unsigned int>(id)) {   
+                    switch (id) {
                     case T_IDENTIFIER:
-                    // test identifier characters for validity (throws if 
+                    // test identifier characters for validity (throws if
                     // invalid chars found)
                         if (!boost::wave::need_no_character_validation(language)) {
                             using boost::wave::cpplexer::impl::validate_identifier_name;
-                            validate_identifier_name(token_val, 
-                                pos.get_line(), pos.get_column(), pos.get_file()); 
+                            validate_identifier_name(token_val,
+                                pos.get_line(), pos.get_column(), pos.get_file());
                         }
                         break;
 
-                    case T_STRINGLIT:
+                    case T_EXTCHARLIT:
+                    case T_EXTSTRINGLIT:
+                    case T_EXTRAWSTRINGLIT:
+                        id = token_id(id & ~AltTokenType);
+                        BOOST_FALLTHROUGH;
+
                     case T_CHARLIT:
-                    // test literal characters for validity (throws if invalid 
+                    case T_STRINGLIT:
+                    case T_RAWSTRINGLIT:
+                    // test literal characters for validity (throws if invalid
                     // chars found)
                         if (boost::wave::need_convert_trigraphs(language)) {
                             using boost::wave::cpplexer::impl::convert_trigraphs;
-                            token_val = convert_trigraphs(token_val); 
+                            token_val = convert_trigraphs(token_val);
                         }
                         if (!boost::wave::need_no_character_validation(language)) {
                             using boost::wave::cpplexer::impl::validate_literal;
-                            validate_literal(token_val, 
-                                pos.get_line(), pos.get_column(), pos.get_file()); 
+                            validate_literal(token_val,
+                                pos.get_line(), pos.get_column(), pos.get_file());
                         }
                         break;
-                        
+
                     case T_LONGINTLIT:  // supported in C99 and long_long mode
                         if (!boost::wave::need_long_long(language)) {
                         // syntax error: not allowed in C++ mode
                             BOOST_WAVE_LEXER_THROW(
-                                boost::wave::cpplexer::lexing_exception, 
-                                invalid_long_long_literal, value.c_str(), 
-                                pos.get_line(), pos.get_column(), 
+                                boost::wave::cpplexer::lexing_exception,
+                                invalid_long_long_literal, value.c_str(),
+                                pos.get_line(), pos.get_column(),
                                 pos.get_file().c_str());
                         }
                         break;
 
-#if BOOST_WAVE_SUPPORT_INCLUDE_NEXT != 0
                     case T_PP_HHEADER:
                     case T_PP_QHEADER:
                     case T_PP_INCLUDE:
                     // convert to the corresponding ..._next token, if appropriate
                         {
-                        // Skip '#' and whitespace and see whether we find an 
+#if BOOST_WAVE_SUPPORT_INCLUDE_NEXT != 0
+                        // Skip '#' and whitespace and see whether we find an
                         // 'include_next' here.
                             typename string_type::size_type start = value.find("include");
                             if (0 == value.compare(start, 12, "include_next", 12))
                                 id = token_id(id | AltTokenType);
+#endif // BOOST_WAVE_SUPPORT_INCLUDE_NEXT != 0
                             break;
                         }
-#endif // BOOST_WAVE_SUPPORT_INCLUDE_NEXT != 0
 
                     case T_EOF:
-                    // T_EOF is returned as a valid token, the next call will 
+                    // T_EOF is returned as a valid token, the next call will
                     // return T_EOI, i.e. the actual end of input
                         at_eof = true;
                         token_val.clear();
                         break;
-                        
+
                     case T_OR_TRIGRAPH:
                     case T_XOR_TRIGRAPH:
                     case T_LEFTBRACE_TRIGRAPH:
@@ -672,29 +762,30 @@ public:
                         }
                         break;
                     }
-                    
+
+                    result = token_type(id, token_val, pos);
 #if BOOST_WAVE_SUPPORT_PRAGMA_ONCE != 0
-                    return guards.detect_guard(token_type(id, token_val, pos));
+                    return guards.detect_guard(result);
 #else
-                    return token_type(id, token_val, pos);
+                    return result;
 #endif
                 }
-            
+
             // skip the T_CONTLINE token
             } while (true);
         }
-        return token;   // return T_EOI
+        return result = token_type();   // return T_EOI
     }
-    
-    void set_position(PositionT const &pos) 
-    { 
+
+    void set_position(PositionT const &pos) BOOST_OVERRIDE
+    {
         // set position has to change the file name and line number only
-        first.get_position().set_file(pos.get_file()); 
-        first.get_position().set_line(pos.get_line()); 
+        first.get_position().set_file(pos.get_file());
+        first.get_position().set_line(pos.get_line());
     }
 
 #if BOOST_WAVE_SUPPORT_PRAGMA_ONCE != 0
-    bool has_include_guards(std::string& guard_name) const 
+    bool has_include_guards(std::string& guard_name) const BOOST_OVERRIDE
         { return guards.detected(guard_name); }
 #endif
 
@@ -703,7 +794,7 @@ private:
     iterator_type last;
     boost::wave::language_support language;
     static lexer::lexer<IteratorT, PositionT> lexer;   // needed only once
-    
+
     bool at_eof;
 
 #if BOOST_WAVE_SUPPORT_PRAGMA_ONCE != 0
@@ -717,24 +808,24 @@ lexer::lexer<IteratorT, PositionT> slex_functor<IteratorT, PositionT>::lexer;
 ///////////////////////////////////////////////////////////////////////////////
 //
 //  The 'new_lexer' function allows the opaque generation of a new lexer object.
-//  It is coupled to the iterator type to allow to decouple the lexer/iterator 
+//  It is coupled to the iterator type to allow to decouple the lexer/iterator
 //  configurations at compile time.
 //
-//  This function is declared inside the cpp_slex_token.hpp file, which is 
+//  This function is declared inside the cpp_slex_token.hpp file, which is
 //  referenced by the source file calling the lexer and the source file, which
-//  instantiates the lex_functor. But is is defined here, so it will be 
-//  instantiated only while compiling the source file, which instantiates the 
+//  instantiates the lex_functor. But it is defined here, so it will be
+//  instantiated only while compiling the source file, which instantiates the
 //  lex_functor. While the cpp_slex_token.hpp file may be included everywhere,
 //  this file (cpp_slex_lexer.hpp) should be included only once. This allows
-//  to decouple the lexer interface from the lexer implementation and reduces 
+//  to decouple the lexer interface from the lexer implementation and reduces
 //  compilation time.
 //
 ///////////////////////////////////////////////////////////////////////////////
 
 ///////////////////////////////////////////////////////////////////////////////
-//  
+//
 //  The new_lexer_gen<>::new_lexer function (declared in cpp_slex_token.hpp)
-//  should be defined inline, if the lex_functor shouldn't be instantiated 
+//  should be defined inline, if the lex_functor shouldn't be instantiated
 //  separately from the lex_iterator.
 //
 //  Separate (explicit) instantiation helps to reduce compilation time.
@@ -745,16 +836,16 @@ lexer::lexer<IteratorT, PositionT> slex_functor<IteratorT, PositionT>::lexer;
 #define BOOST_WAVE_SLEX_NEW_LEXER_INLINE
 #else
 #define BOOST_WAVE_SLEX_NEW_LEXER_INLINE inline
-#endif 
+#endif
 
 template <typename IteratorT, typename PositionT>
 BOOST_WAVE_SLEX_NEW_LEXER_INLINE
 lex_input_interface<slex_token<PositionT> > *
 new_lexer_gen<IteratorT, PositionT>::new_lexer(IteratorT const &first,
-    IteratorT const &last, PositionT const &pos, 
+    IteratorT const &last, PositionT const &pos,
     boost::wave::language_support language)
 {
-    return new slex_functor<IteratorT, PositionT>(first, last, pos, 
+    return new slex_functor<IteratorT, PositionT>(first, last, pos,
         language);
 }
 
@@ -765,5 +856,5 @@ new_lexer_gen<IteratorT, PositionT>::new_lexer(IteratorT const &first,
 }   // namespace cpplexer
 }   // namespace wave
 }   // namespace boost
-     
-#endif // !defined(SLEX_LEXER_HPP_5E8E1DF0_BB41_4938_B7E5_A4BB68222FF6_INCLUDED)
+
+#endif // !defined(BOOST_SLEX_LEXER_HPP_5E8E1DF0_BB41_4938_B7E5_A4BB68222FF6_INCLUDED)
